from clink.com.type import Component


class AppErrHandler(Component):
    pass


class AppReqHandler(Component):
    pass


class AppResHandler(Component):
    pass


class Controller(Component):
    pass


class Service(Component):
    pass
